==> qmpdclient-1.0.6.2.tar.gz: qmpdclient-1.0.6.2/src/dynamicplaylist.cpp <==
/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "dynamicplaylist.h"
#include "mpd.h"
#include "mpdsong.h"
#include <QTime>

DynamicPlaylist::DynamicPlaylist(QObject *parent) : QObject(parent), m_lastPlaying(0) {
	setObjectName("autoplay");
	m_mpd = MPD::instance();
	m_config = Config::instance();
	connect(m_mpd, SIGNAL(playingSongUpdated(MPDSong *)), this, SLOT(autoAdd(MPDSong *)));
	connect(m_mpd, SIGNAL(playingSongUpdated(MPDSong *)), this, SLOT(autoRemove(MPDSong *)));
	connect(m_mpd, SIGNAL(playlistUpdated(MPDSongList)), this, SLOT(playlistUpdated(MPDSongList)));
	connect(m_mpd, SIGNAL(songsUpdated(MPDSongList)), this, SLOT(songsUpdated(MPDSongList)));
}

void DynamicPlaylist::songsUpdated(MPDSongList fs) {
	m_songs = fs;
}

void DynamicPlaylist::playlistUpdated(MPDSongList fs) {
	m_playlist = fs;
}

void DynamicPlaylist::autoAdd(MPDSong *song) {
	if (!song || !m_config->autoAddSongs() || m_playlist.isEmpty())
		return;

	MPDSong *nthLast = m_playlist.at(m_playlist.size() - (1 + m_config->autoAddPos()));

	if (song->pos() < nthLast->pos() || m_songs.isEmpty())
		return;

	MPDSongList songsToAdd;
	srand(QTime(0,0,0).secsTo(QTime::currentTime()));
	for (int i=0; i<m_config->autoAddCount(); i++)
		songsToAdd << m_songs.at(rand() % m_songs.size());
	m_mpd->addSongs(songsToAdd);
}

void DynamicPlaylist::autoRemove(MPDSong *song) {
	if (!song || !m_config->autoRemoveSongs()) {
		m_lastPlaying = song;
		return;
	}

	// Find song's new ID.
	foreach(MPDSong *s, m_playlist) {
		if (m_lastPlaying && s->url() == m_lastPlaying->url()) {
			MPDSongList songs;
			songs << s;
			m_mpd->removeSongs(songs);
			break;
		}
	}
	m_lastPlaying = song;
}

==> qmpdclient-1.0.7.tar.gz: qmpdclient-1.0.7/src/dynamicplaylist.cpp <==
/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "dynamicplaylist.h"
#include "mpd.h"
#include <QTime>

DynamicPlaylist::DynamicPlaylist(QObject *parent) : QObject(parent) {
	setObjectName("autoplay");
	m_mpd = MPD::instance();
	m_config = Config::instance();
	connect(m_mpd, SIGNAL(playingSongUpdated(MPDSong)), this, SLOT(autoAdd(MPDSong)));
	connect(m_mpd, SIGNAL(playingSongUpdated(MPDSong)), this, SLOT(autoRemove(MPDSong)));
	connect(m_mpd, SIGNAL(playlistUpdated(MPDSongList)), this, SLOT(playlistUpdated(MPDSongList)));
	connect(m_mpd, SIGNAL(songsUpdated(MPDSongList)), this, SLOT(songsUpdated(MPDSongList)));
}

void DynamicPlaylist::songsUpdated(const MPDSongList &fs) {
	m_songs = fs;
}

void DynamicPlaylist::playlistUpdated(const MPDSongList &fs) {
	m_playlist = fs;
}

void DynamicPlaylist::autoAdd(const MPDSong &song) {
	if (song.isNull() || !m_config->autoAddSongs() || m_playlist.isEmpty())
		return;
	int n = m_playlist.size() - (1 + m_config->autoAddPos());
	if (n < 0)
		return;
	MPDSong nthLast = m_playlist.at(n);

	if (song.pos() < nthLast.pos() || m_songs.isEmpty())
		return;

	MPDSongList songsToAdd;
	qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));
	for (int i=0; i<m_config->autoAddCount(); i++)
		songsToAdd << m_songs.at(qrand() % m_songs.size());
	m_mpd->addSongs(songsToAdd);
}

void DynamicPlaylist::autoRemove(const MPDSong &song) {
	if (song.isNull() || !m_config->autoRemoveSongs()) {
		m_lastPlaying = song;
		return;
	}

	// Find song's new ID.
	foreach(MPDSong s, m_playlist) {
		if (!m_lastPlaying.isNull() && s.url() == m_lastPlaying.url()) {
			MPDSongList songs;
			songs << s;
			m_mpd->removeSongs(songs);
			break;
		}
	}
	m_lastPlaying = song;
}
